package edu.dhbw.pv;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

/** 
 * ParallelMatrixTrans with ExecutorService.
 * @author Florian Baier.
 */
public class ParallelMatrixTransExecutor extends Matrix {
        
    public int nthreads;
        
    public ParallelMatrixTransExecutor(int n, int m, int nthreads) {
        super(n,m);
        this.nthreads = nthreads;
    }
    public ParallelMatrixTransExecutor(double [][] v, int nthreads) {
        super(v);
        this.nthreads = nthreads;
    }
    
    public Matrix multiply(Matrix B) {
        double[][] erg = new double[val.length][B.val[0].length];
        ExecutorService exSer = Executors.newFixedThreadPool(nthreads);
        B = B.transpose();
        Future[] f = new Future[val.length];
        for (int i = 0;i < val.length; i++ ) {
            f[i] = exSer.submit(new ZeilenMultiplikationTransExecutor(erg, val, B.val, i));
        }
        for(int i = 0; i < val.length; i++){                      
            try {
                f[i].get();
            } catch (InterruptedException e) {
                e.printStackTrace();
            } catch (ExecutionException e) {
                e.printStackTrace();
            }
        }
        exSer.shutdown();
        return new Matrix(erg);
    } 
}

class ZeilenMultiplikationTransExecutor extends Thread {
    double[][] C;
    double[][] A;
    double[][] B;
    int i;
    public ZeilenMultiplikationTransExecutor(double[][] cp, double[][] ap, double[][] bp, int ip) {
        C = cp; A = ap; B = bp; i = ip;      
    }
    public void run() {
        for (int j = 0; j < C[0].length; j++ ) {
            double s = 0.0;
            for (int k = 0; k < A[0].length; k++ ) {
                s += A[i][k] * B[j][k]; // war k,j
            }
            C[i][j] = s;
        }
    }
}

